<!----------------------------------------------------------------------------->
# Add a new project using the CI pipeline
<!----------------------------------------------------------------------------->

The CI script infrastructure allow automatical pipeline builds and integration
using the complete Yocto build system.

To add a new project several steps are needed to make this work together:


## Create the project

Just use the new project button in Gitlab and prepare the project with some
source code.

## Configure the new project

In the Gitlab UI use the settings:

`Settings->Repository`:

* *Default branch*: Set the Default branch to have the name of the main Yocto
  version to which it corresponds, e.g. `kirkstone`.

* *Protected branch*: Add at least the default branch to the 'protected branches' list.
  Select `Allowed to merge` to `Maintainers` and `Allowed to push` to `No one`.

`Settings->Merge requests`:

* *Merge method*: Set to `Fast-forward merge`

* *Squash commits when merging*: Set to `Do not allow`

* *Merge checks*: Set to `Pipelines must succeed`


`Settings->CI/CD`:

* *Variables*: Add a variable `INTEGRATION` that specifies which branch of this
  project should be integrated into which branch of the Yocto manifest.

  Example: `kirkstone:edgehog/seco-manifest:kirkstone`
  
  * `kirkstone`: The default branch of the new project.
  * `edgehog/seco-manifest`: The path to the manifest of the Yocto project.
                              This might differ for custom layers.
  * `kirkstone`: The branch of the Yocto manifest to be used for the
                 integration.

  There may be multiple lines in the `INTEGRATION` variable to integrate into
  several manifests or branches.

  **NOTE**: In case the project that is being added is referred to inside a `SRCREV.conf` file
  of a meta-layer project, the `INTEGRATION` variable shall contain as a middle variable the 
  path to the meta-layer inside GitLab, e.g. `edgehog/layers/seco/meta-seco-imx`. The last 
  variable shall be the the branch of Yocto meta-layer to be used for the integration.

* *Variables*: Add a variable `MANIFEST_FILE` that contains the path to the manifest file in the 
  manifest repository. For example, for the `meta-seco-edgehog-embedded` project, this variable is
  set to `distro/seco-distro.xml`.s
## Add the new project to the Yocto project

### Project referenced by recipe

This part is relevant if the new repo should be used by a recipe as new package.

Create a new recipe in one of the layers used by the Yocto project like
`<new-project.bb>`:

```
SUMMARY = "..."
DESCRITPTION = "..."
HOMEPAGE = "https://git.seco.com/seco-ne/<path to the new project>"

LICENSE = "Apache-2.0"
LIC_FILES_CHKSUM = "file://${COMMON_LICENSE_DIR}/Apache-2.0;md5=89aea4e17d99a7cacdbeed46a0096b10"

SRCREV = "${AUTOREV}"
SRC_URI = " \
  git://git.seco.com/edgehog/<path to the new project>.git;protocol=https;branch=main;nobranch=1 \
"
```

Push this to the layers repo.

### Add a reference in SRCREV.conf

In the 'layer' directory, use the target branch and modify the `conf/SRCREV.conf`.
It needs a new line for the new project like:

```
SRCREV:pn-<new-project> = "<current hash>"
```

In particular, this procedure is necessary for the projects contained inside the `bsp`
group, since we need to integrate the U-Boot, Kernel and Firmware tools versions. To do 
so, we need to create for each `meta-seco-<silicon-vendor>` layer a SRCREV.conf file 
containing the references to the Board Support Package (BSP) repositories.

## Add the CI code to the new project

### Add submodule .gitlab-ci

The submodule needs to be added with a relative path.

```
git submodule add ../../../infrastructure/gitlab-ci.git .gitlab-ci
```

### Add the local .gitlab-ci.yml

```
---
# ---------------------------------------------------------------------------------------
# Include the default CI steps from the gitlab-ci repo
# ---------------------------------------------------------------------------------------
include:
  - project: '${CI_PROJECT_ROOT_NAMESPACE}/Yocto/infrastructure/gitlab-ci'
    ref: 31e5a427d57e2d3cac971a32d18e6274932adb99
    file: 'manifest-integration.yml'

variables:
  GITLAB_CI_REVISION: 31e5a427d57e2d3cac971a32d18e6274932adb99
  BB_RECIPE_NAME: <recipe name, like emc-test-suite, without the '.bb'>
```

The `BB_RECIPE_NAME` needs to be adapted, as it is later used to modify the SRCREV.

<!----------------------------------------------------------------------------->
## Custom Variables 
<!----------------------------------------------------------------------------->

For custom projects inside the Edgehog group, the same procedure described above applies 
to add CI/CD steps in the dedicated Yocto projects. However, another set of variables needs
to be defined at project or YAML global variables level (depending on the necessary priority
level) in the custom manifest repository:<>

* `CUSTOM`  
  Name of the custom project
* `PROCESSOR`  
  Name of the processor for the custom board, e.g. `rk3568`
* `MACHINE`  
  Name of the custom machine (used in the SW artifacts naming)
* `DEFCONFIG`  
  Name of the Edgehog Things defconfig file used by seco-setup.sh script, e.g. `seco_sbc_c20_edgehog`
* `BUILD_DIR`  
  Name of the Edgehog Things build directory created by the seco-setup.sh script, e.g. `build_c20`
* `KERNEL_FILE`  
  Name of the Kernel file, e.g. `Image`
* `UBOOT_FILE`  
  Space separated list of U-Boot files, e.g. `uboot.img idblock.img`
* `AZURE_PATH`  
  Path to save the SW artifacts to in Azure container
* `CHANGELOG_PROJECTS`  
  List of the projects to be considered in the custom changelog.
* `RETRIGGER_GROUP`  
  Path of the custom group in GitLab.
