<!----------------------------------------------------------------------------->
# Manifest pipeline
<!----------------------------------------------------------------------------->

The pipeline running in the manifest repository is the one that does the real
work, like building images, deploying image to Azure Cloud storage and triggering
tests in the LAVA test rack. The pipeline running in the manifest repository is
 described in the `manifest-pipeline-yocto.yml` [file][1].

[1]:../manifest-pipeline-yocto.yml

<!----------------------------------------------------------------------------->
## Dynamic Pipeline Generation
<!----------------------------------------------------------------------------->

### Parent/child pipelines

We support multiple machines/architectures, and for each one of them we need the
same set of build, deploy and test jobs. To simplify the definition
of all these jobs without lots of copy and paste actions, and with the
possibility to configure the actual build images, we are using [GitLab's
dynamic-child-pipeline feature][2].

[2]: https://docs.gitlab.com/ee/ci/pipelines/parent_child_pipelines.html#dynamic-child-pipeline-example

There is a `generate-build-pipeline` job that creates a YAML file containing
the pipeline with all the jobs for each machine, based on its input variables
`MACHINES`, `DISTRO` and `GRAPHIC_BACKEND`:

* `MACHINES`  
  Space separated list of machines to build for, e.g. "c12 c20"

* `DISTRO`  
  Space separated list of Edgehog OS distros to build for, e.g. "things embedded-base embedded-full"

* `GRAPHIC_BACKEND`  
  Space separated list of graphical backends to build for, e.g. "wayland x11"

This is done using the Python script [`render_jinja2_template.py`][3], which
renders the template [`build-pipeline-yocto.yml.jinja2`][4] to
`build-pipeline.yml`.

[3]: ../scripts/render_jinja2_template.py
[4]: ../build-pipeline-yocto.yml.jinja2

See the following screenshot for how GitLab displays this pipeline:

![Manifest's parent child pipeline](manifest-parent-child.png)

<!----------------------------------------------------------------------------->
## Triggers
<!----------------------------------------------------------------------------->

The build in the manifest repository is currently triggered both on a weekly 
basis (on Friday evenings) on the `MASTER_BRANCH` branch and asynchronously 
whenever a merge request is opened on a project or on the repository containing
CI/CD scripts. Eventually the asynchronous build can be skipped in case the
`skip build` label is specified when opening the merge request. 

**NOTE**: In the current implementation, the build/deploy/test stages are left to
be triggered manually whenever a MR is created in order to not overload the build
server.

The `deploy` and `notify` stage jobs are currently triggered only on scheduled 
pipelines.

<!----------------------------------------------------------------------------->
## Build jobs
<!----------------------------------------------------------------------------->

The build stage generic description is present in the `manifest-pipeline-yocto.yml`
file while the variables which differs from board to board are imported from the 
the `build-pipeline-yocto.yml.jinja2` [template][4]. the template file already 
contains all SECO hardware to be included into Edgehog, so that in order to 
include another board within the compiled one it will then be necessary to include 
it into the `MACHINES` variable list.

The build jobs either compile the Egdehog image and the RAUC bundle for Edgehog 
Things distro.

<!----------------------------------------------------------------------------->
## Deploy and upload jobs
<!----------------------------------------------------------------------------->

The artifacts from the previous stage are made available to the `deploy` stage 
which allows to rename, assign a tag and push to our Azure blob storage containers
all the relevant artifacts from the `build` stage, i.e. U-Boot, Kernel, `.wic`
image, etc. The upload to Azure simply takes advantage of the three [variables
defined at group level][5]:

* `AZURE_STORAGE_ACCOUNT`: the name of the blob storage account software artifacts shall be uploaded to.
* `AZURE_CONTAINER_NAME`: the name of the blob storage container used to store software artifacts.
* `AZURE_STORAGE_SAS_TOKEN`: the [SAS token][6] associated to the corresponding Azure container.

[5]:https://git.seco.com/groups/edgehog/-/settings/ci_cd
[6]:https://learn.microsoft.com/it-it/azure/cognitive-services/translator/document-translation/create-sas-tokens?tabs=Containers

<!----------------------------------------------------------------------------->
## Changelog 
<!----------------------------------------------------------------------------->

Every time a build is executed on the manifest, the changelog is also generated 
from one of the scripts accessing the GitLab API in the [scripts folder][7]. In particular, 
the job makes it possible to obtain a file in `.md` format containing the titles 
and URL references of all the MRs merged into the various meta-layers. This gives an idea of the 
various major changes between different versions of Edgehog OS. 

[7]:https://git.seco.com/edgehog/infrastructure/gitlab-ci/-/tree/master/scripts

<!----------------------------------------------------------------------------->
## Notify Job
<!----------------------------------------------------------------------------->

Once all jobs associated to the `build` and `deploy` stages have finished, the 
`notify` stage allows to collect the results from all previous jobs (and the link 
to the artifacts) and to send them to the dedicated channel in Microsoft Teams 
Edgehog group. The notification is allowed by using an [Incoming WebHook][8], 
saved as a group variable.

[8]:https://learn.microsoft.com/en-us/microsoftteams/platform/webhooks-and-connectors/how-to/add-incoming-webhook
<!----------------------------------------------------------------------------->
## Release 
<!----------------------------------------------------------------------------->

Only when the tag is executed will a final job also be executed that allows a [GitLab 
release][9] associated with that tag to be created. This job makes it possible 
to associate with the GitLab release all the links to the SW artifacts produced by
the build jobs in such a way that they are easily accessible, and allows the 
previously generated changelog file to be associated with the release description.

[9]:https://docs.gitlab.com/ee/user/project/releases/
